SUBROUTINE LAI_ANALYSIS(KNT, KLIN, KCOL, PFRAC, PLAI_ECO, PLAI_SAT, PLAI_SAT_ERR, &
                        PLAI_GUESS, PERR_GUESS, PLAI_A, PERR_A, PLAI_A_MEAN)

IMPLICIT NONE

INCLUDE 'mpif.h'

INTEGER, INTENT(IN) :: KNT ! 12
INTEGER, INTENT(IN) :: KLIN
INTEGER, INTENT(IN) :: KCOL

REAL (KIND=4), DIMENSION(KNT,KLIN,KCOL), INTENT(IN) :: PFRAC ! FRAC. TILE
REAL (KIND=4), DIMENSION(KNT,KLIN,KCOL), INTENT(IN) :: PLAI_ECO ! ECOCLIMAP LAI. TILE

REAL (KIND=4), DIMENSION(KLIN,KCOL), INTENT(IN) :: PLAI_SAT   ! OBSERVED LAI FROM SAT AND QUAL FLAG- DAY J
REAL (KIND=4), INTENT(IN) :: PLAI_SAT_ERR
!
REAL (KIND=4), DIMENSION(KNT,KLIN,KCOL), INTENT(IN) :: PLAI_GUESS ! GUESS  LAI TILE- DAY OF THE LAST ANALYSIS

REAL (KIND=4), DIMENSION(KNT,KLIN,KCOL), INTENT(IN) :: PERR_GUESS

REAL (KIND=4), DIMENSION(KNT,KLIN,KCOL), INTENT(INOUT) :: PLAI_A ! LAI TILE ANALYSED -DAY J
REAL (KIND=4), DIMENSION(KNT,KLIN,KCOL), INTENT(INOUT) :: PERR_A  ! COVAR TILE ANALYSED -DAY J
REAL (KIND=4), DIMENSION(KLIN,KCOL), INTENT(INOUT) :: PLAI_A_MEAN   ! LAI ANALYSED - DAY J

REAL (KIND=4), DIMENSION(KNT) :: ZFRAC

!REAL (KIND=4), INTENT(IN) :: ZLAI_MOD   ! ECOCLIMAP LAI - DAY J
!REAL, PARAMETER ::ZSIG_LAI_M=.5  ! ERROR OF LAI - DAY J
REAL, PARAMETER :: XT = 10.   !30.  ! TIME SCALE OF THE ANALYSIS PROCESS (MONTHLY)
REAL, PARAMETER :: XTC = 30. !20.  ! CARACTERISTIC TIME SCALE OF LAI EVOLUTION

REAL, DIMENSION(:,:), ALLOCATABLE :: ZA, ZB, ZR, ZH, ZK, ZT1, ZT2
REAL, DIMENSION(:), ALLOCATABLE :: ZY, ZX

INTEGER, DIMENSION(KNT,KNT) :: II_N 

REAL :: ZALPHA
INTEGER :: JT, JJ, IERR, JC, JL, NRANK, INFOMPI

! INTERFACE FOR MATRIX INVERSION
INTERFACE
  FUNCTION INVERS(PM)
    REAL, DIMENSION(:,:), INTENT(IN)  :: PM
    REAL, DIMENSION(SIZE(PM,1),SIZE(PM,1)) :: INVERS
  END FUNCTION INVERS
END INTERFACE

CALL MPI_COMM_RANK(MPI_COMM_WORLD,NRANK,INFOMPI)

IF(.NOT. ALLOCATED(ZX)) ALLOCATE(ZX(KNT))
IF(.NOT. ALLOCATED(ZB)) ALLOCATE(ZB(KNT,KNT))
IF(.NOT. ALLOCATED(ZA)) ALLOCATE(ZA(KNT,KNT))

IF(.NOT. ALLOCATED(ZY)) ALLOCATE(ZY(KNT+1))

IF(.NOT. ALLOCATED(ZH)) ALLOCATE(ZH  (KNT+1,KNT))
IF(.NOT. ALLOCATED(ZK)) ALLOCATE(ZK  (KNT,KNT+1))
IF(.NOT. ALLOCATED(ZT2)) ALLOCATE(ZT2(KNT,KNT+1))

IF(.NOT. ALLOCATED(ZR)) ALLOCATE(ZR  (KNT+1,KNT+1))
IF(.NOT. ALLOCATED(ZT1)) ALLOCATE(ZT1(KNT+1,KNT+1))

DO JL = 1,KLIN
  !print*,JL,KLIN  
  DO JC = 1,KCOL
    !if (jl==1.and.jc==1168) print*,'debut ',PLAI_SAT(JL,JC),PFRAC(:,JL,JC)
    IF ( PLAI_SAT(JL,JC)>=0.05.AND.PLAI_SAT(JL,JC)<=7.0 .AND. &
         SUM(PFRAC(:,JL,JC))/=0. ) THEN

      !if (jl==7.and.jc==1783.and.nrank==14) print*,'cas1'
      !print*,'cas1'
      !INITIALISE
      ZFRAC(4:KNT) = PFRAC(4:KNT,JL,JC)  !/SUM(PFRAC(4:KNT,JL,JC))
      ZFRAC(1:3) = 0.
      !ZFRAC(:) = PFRAC(:,JL,JC) !/ SUM(PFRAC(:,JL,JC))
      
      !ALLOCATE X
      ZX(:) = PLAI_GUESS(:,JL,JC)   !! DONNEES GUESS TILE SUR PIXEL
      !if (jc==1885) print*,'x',ZX

      !ALLOCATE Y
      ZY(1:KNT) = PLAI_ECO(:,JL,JC) !! DONNEES ECOCLIMAP SUR PIXEL
      ZY(KNT+1) = PLAI_SAT(JL,JC)  !! DONNEES SAT SUR PIXEL
      !if (jc==1885) print*,'y',ZY
 
      !ALLOCATE H
      ZH(:,:) = 0.
      DO JT =1,KNT
        ZH(JT,JT) = 1
        ZH(KNT+1,JT) = ZFRAC(JT)
      ENDDO
      !if (sum(zh(knt+1,:))<0.999.or.sum(zh(knt+1,:))>1.001) print*,'HHHHHHHHHHHHHHhhhh ',JL,JC,sum(zh(knt+1,:))
      !if (jc==1885) print*,'h',ZH(KNT+1,:)

      !ALLOCATE R
      ZR(:,:) = 0.
      DO JT = 1,KNT
        ZR(JT,JT) = 1. !!ZSIG_LAI_M**2 # ERREUR SUR ECOCLIMAP
      ENDDO
      ZR(KNT+1,KNT+1) = PLAI_SAT_ERR**2
      !if (jc==1885) print*,'r ',ZR(KNT+1,KNT+1)

      !ALLOCATE II_N
      II_N(:,:) = 0
      DO JT = 1,KNT
        II_N(JT,JT) = 1
      ENDDO
      !CALCULATE B
      ZB(:,:)=0.
      DO JT=1,KNT
        DO JJ=1,KNT

          IF (JT .NE. JJ) THEN
            ZB(JT,JJ) = 0.
          ELSE

            IF ( ZX(JT).GT.7. ) THEN  
              ! NO GUESS
              !ZALPHA=1.+MAX(0.1,PFRAC(JT))
              ZALPHA = 1. + MIN(1.,ZFRAC(JT))
            ELSE
              !ZALPHA=1.+MAX(0.1,PFRAC(JT))
              ZALPHA = 1. + MIN(1.,ZFRAC(JT))
            ENDIF   

            !print*,jt,jl,jc,perr_guess(jt,jl,jc)
            ZB(JT,JJ) = PERR_GUESS(JT,JL,JC) * ZALPHA * (2**(2.*XT/XTC))
            if (jl==7.and.jc==1783.and.nrank==14) then
        !      print*,'ZB ',JT,ZB(JT,JJ)
            endif

            ! SECURITY LIMIT
            IF (ZFRAC(JT) .EQ. 0.)   ZB(JT,JJ) = 0.000001
            IF (ZB(JT,JJ) .GE. 100000000.) ZB(JT,JJ) = 100000000.
            IF (ZB(JT,JJ) .LE. -100000000.)   ZB(JT,JJ) = -100000000.
            !IF (ZB(JT,JJ) .LE. 0.000001)   ZB(JT,JJ) = 0.000001
          ENDIF

        ENDDO
        !      if (jc==1885) print*,'b ',JT,ZB(JT,JJ)
      ENDDO
     
      ! SOIL AND LAIETATION ALBEDO ANALYSES

      ! (KNT+1,KNT+1) = (KNT+1,KNT) * ( (KNT,KNT) * (KNT,KNT+1) ) + (KNT+1,KNT+1)
      ZT1(:,:) = INVERS( MATMUL( ZH(:,:), MATMUL(ZB(:,:), TRANSPOSE(ZH(:,:))) ) + ZR(:,:)  )
 
      ! (KNT,KNT+1) = (KNT,KNT+1) * (KNT+1,KNT+1)
      ZT2(:,:) = MATMUL( TRANSPOSE( ZH(:,:)), ZT1(:,:) )

      ! (KNT,KNT+1) = (KNT,KNT) * (KNT,KNT+1)
      ZK(:,:) = MATMUL( ZB(:,:), ZT2(:,:) )
      if (jl==7.and.jc==1783.and.nrank==14) then
      !   print*,'zk5',zk(5,:)
      !   print*,'zk12 ',zk(12,:)
      !   print*,'zh5',zh(:,5)
      !   print*,'zh12 ',zh(:,12)
      !   print*,'zk',zk(5,5),zk(12,12),zh(14,14)
      !   print*,'zh',zh(5,5),zh(12,12),zk(14,14)
      !   print*,'zb',zb(5,5),zb(12,12),zb(14,14)
       endif

      ! (KNT,KNT) = ( (KNT,KNT) - (KNT,KNT+1) * (KNT+1,KNT) ) * (KNT,KNT)
      ZA(:,:)=MATMUL(II_N(:,:) - MATMUL( ZK(:,:), ZH(:,:) ), ZB(:,:)) 
      !ZA(:,:)=II_N(:,:) - MATMUL( ZK(:,:), ZH(:,:) )
      if (jl==7.and.jc==1783.and.nrank==14) then
       !       print*,'za',za(5,5),za(12,12),za(14,14)
      endif
      !ZA(:,:)=MATMUL(ZA(:,:), ZB(:,:)) 
if (jl==7.and.jc==1783.and.nrank==14) then
       !       print*,'za2',za(5,5),za(12,12),za(14,14)
      endif


      ! SECURITY FOR VARIANCE & LAI VALUES 
      ! KNT = KNT + ( (KNT,KNT+1) * ( (KNT+1) - (KNT+1,KNT) * KNT ))
      ZX(:) = ZX(:) + MATMUL(ZK(:,:), ( ZY(:) - MATMUL( ZH(:,:), ZX(:) ) ) )
      !PRINT *, 'VAL MAX A=',MAXVAL(A) , 'VAL MIN A=',MINVAL(A)
 
      WHERE (ZA(:,:) .LE. -1000.**2) ZA(:,:) = -1000.**2        
      WHERE (ZA(:,:) .GE.  1000.**2) ZA(:,:) = 1000.**2        
      WHERE (ZX(:) .LT. 0.) ZX(:) = 0.

      ! OUTPUT
      !LAI TILE ANALYSED <--- X
      !Y ECO TILES & SAT <--- Y(KNT)& Y(KNT+1)

      ! CALCULATE LAI ANA TOT PIXEL
      PLAI_A_MEAN(JL,JC) = 0.
      DO JT = 1,KNT
        ! SECURITY TESTS - SHOULD PROBABLY NOT BE ENOUGH... !!!!
        !X(I)=MIN(X(I), 0.99)
        !X(I)=MAX(X(I), 0.01)
        PLAI_A_MEAN(JL,JC) = PLAI_A_MEAN(JL,JC) + ZX(JT) * PFRAC(JT,JL,JC)
      ENDDO

      ! SECURITY TESTS - SHOULD PROBABLY NOT BE ENOUGH... !!!!
      DO JT = 1,KNT
        IF (PLAI_A_MEAN(JL,JC) .GT. 7.) THEN
          ZX(JT) = ZX(JT) * 7. / PLAI_A_MEAN(JL,JC)
        ENDIF
      ENDDO

      PLAI_A_MEAN(JL,JC) = MIN(PLAI_A_MEAN(JL,JC), 7.)       
      PLAI_A_MEAN(JL,JC) = MAX(PLAI_A_MEAN(JL,JC), 0.)

      !WHERE (X(:) .LT. 10 .AND. X(:) .GT. 0.99)
      !   LAI_A(:,:)= 0.99      
      !END WHERE
      PLAI_A (:,JL,JC) = 0.
      PERR_A(:,JL,JC) = 0.
      DO JT = 1,KNT

        IF(ZX(JT) .LT. 10) ZX(JT) = MIN(ZX(JT), 7.)
        PLAI_A (JT,JL,JC) = ZX(JT)
        PERR_A(JT,JL,JC) = ZA(JT,JT)

      ENDDO
    
    ELSE

      !if (jl==7.and.jc==1783.and.nrank==14) print*,'cas2'

      !if (jl==1.and.jc==1168) print*,'lai_eco ',KNT,PLAI_ECO(:,JL,JC)
      PLAI_A(:,JL,JC) = PLAI_ECO(:,JL,JC)
      PERR_A(:,JL,JC) = PERR_GUESS(:,JL,JC)*(2**(2*XT/XTC))
      WHERE (PERR_A(:,JL,JC).GE.100000000.) PERR_A(:,JL,JC) = 100000000.
      PLAI_A_MEAN(JL,JC) = 0.
      DO JT = 1,KNT
        PLAI_A_MEAN(JL,JC) = PLAI_A_MEAN(JL,JC) + PLAI_A(JT,JL,JC) * PFRAC(JT,JL,JC)
      ENDDO
      !if (jl==1.and.jc==1168) print*,'lai_a ',PLAI_A_MEAN(JL,JC)
      
    ENDIF

  ENDDO
ENDDO
!print*,'minmax ',minval(PLAI_A_MEAN(:,:)),maxval(PLAI_A_MEAN(:,:),PLAI_A_MEAN(:,:)/=25.5),&
!        maxloc(PLAI_A_MEAN(:,:),PLAI_A_MEAN(:,:)/=25.5)
!if (any(PERR_A(19,:,:)/=0..AND.PERR_A(19,:,:)/=1585.81360)) &
!        print*,'erra_ ',minval(PERR_A(19,:,:)),maxval(PERR_A(19,:,:),PERR_A(19,:,:)/=1585.81360),&
!        maxloc(PERR_A(19,:,:),PERR_A(19,:,:)/=1585.81360)
!LAI TOT ANALYSED <--- PLAI_A_MEAN 
       
! DEALLOCATE
DEALLOCATE( ZY, ZX, ZA, ZB, ZR, ZH, ZK, ZT1, ZT2, STAT=IERR )

IF (IERR .NE. 0) THEN
  PRINT *,'ERROR WHEN DEALLOCATING VARIABLES PACKAGE LAI!'
  RETURN
END IF        

END SUBROUTINE LAI_ANALYSIS

