MODULE NRUTIL

USE NRTYPE

IMPLICIT NONE

INTERFACE SWAP 
  MODULE PROCEDURE SWAP_I, SWAP_R, SWAP_RV
END INTERFACE

INTERFACE ASSERT_EQ
  MODULE PROCEDURE ASSERT_EQ2, ASSERT_EQ3, ASSERT_EQ4, ASSERT_EQN
END INTERFACE

INTERFACE IMAXLOC
  MODULE PROCEDURE IMAXLOC_R, IMAXLOC_I
END INTERFACE

INTERFACE OUTERPROD
  MODULE PROCEDURE OUTERPROD_R, OUTERPROD_D
END INTERFACE

CONTAINS

SUBROUTINE SWAP_I(KA,KB)
  INTEGER(I4B), INTENT(INOUT) :: KA, KB
  INTEGER(I4B) :: IDUM
  IDUM = KA
  KA = KB
  KB = IDUM
END SUBROUTINE SWAP_I

SUBROUTINE SWAP_R(PA,PB)
  REAL(SP), INTENT(INOUT) :: PA,PB
  REAL(SP) :: ZDUM
  ZDUM = PA
  PA = PB
  PB = ZDUM
END SUBROUTINE SWAP_R

SUBROUTINE SWAP_RV(PA,PB)
  REAL(SP), DIMENSION(:), INTENT(INOUT) :: PA,PB
  REAL(SP), DIMENSION(SIZE(PA)) :: ZDUM
  ZDUM = PA
  PA = PB
  PB = ZDUM
END SUBROUTINE SWAP_RV


FUNCTION ASSERT_EQ2(K1,K2,HSTRING)
  CHARACTER(LEN=*), INTENT(IN) :: HSTRING
  INTEGER, INTENT(IN) :: K1,K2
  INTEGER :: ASSERT_EQ2
  IF (K1 == K2) THEN
    ASSERT_EQ2 = K1
  ELSE
    WRITE (*,*) 'NRERROR: AN ASSERT_EQ FAILED WITH THIS TAG:',HSTRING
    STOP 'PROGRAM TERMINATED BY ASSERT_EQ2'
  END IF
END FUNCTION ASSERT_EQ2

FUNCTION ASSERT_EQ3(K1,K2,K3,HSTRING)
  CHARACTER(LEN=*), INTENT(IN) :: HSTRING
  INTEGER, INTENT(IN) :: K1,K2,K3
  INTEGER :: ASSERT_EQ3
  IF (K1 == K2 .AND. K2 == K3) THEN
    ASSERT_EQ3 = K1
  ELSE
    WRITE (*,*) 'NRERROR: AN ASSERT_EQ FAILED WITH THIS TAG:',HSTRING
    STOP 'PROGRAM TERMINATED BY ASSERT_EQ3'
  END IF
END FUNCTION ASSERT_EQ3

FUNCTION ASSERT_EQ4(K1,K2,K3,K4,HSTRING)
  CHARACTER(LEN=*), INTENT(IN) :: HSTRING
  INTEGER, INTENT(IN) :: K1,K2,K3,K4
  INTEGER :: ASSERT_EQ4
  IF (K1 == K2 .AND. K2 == K3.AND. K3 == K4) THEN
    ASSERT_EQ4 = K1
  ELSE
    WRITE (*,*) 'NRERROR: AN ASSERT_EQ FAILED WITH THIS TAG:',HSTRING
    STOP 'PROGRAM TERMINATED BY ASSERT_EQ4'
  END IF
END FUNCTION ASSERT_EQ4

FUNCTION ASSERT_EQN(KN,HSTRING)
  CHARACTER(LEN=*), INTENT(IN) :: HSTRING
  INTEGER, DIMENSION(:), INTENT(IN) :: KN
  INTEGER :: ASSERT_EQN
  IF (ALL(KN(2:) == KN(1))) THEN
    ASSERT_EQN = KN(1)
  ELSE
     WRITE (*,*) 'NRERROR: AN ASSERT_EQ FAILED WITH THIS TAG:',HSTRING
     STOP 'PROGRAM TERMINATED BY ASSERT_EQN'
  END IF
END FUNCTION ASSERT_EQN

SUBROUTINE NRERROR(HSTRING)
  CHARACTER(LEN=*), INTENT(IN) :: HSTRING
  WRITE (*,*) 'NRERROR: ',HSTRING
  STOP 'PROGRAM TERMINATED BY NRERROR'
END SUBROUTINE NRERROR

FUNCTION IMAXLOC_R(PARR)
  REAL(SP), DIMENSION(:), INTENT(IN) :: PARR
  INTEGER(I4B) :: IMAXLOC_R
  INTEGER(I4B), DIMENSION(1) :: IMAX
  IMAX=MAXLOC(PARR(:))
  IMAXLOC_R = IMAX(1)
END FUNCTION IMAXLOC_R

FUNCTION IMAXLOC_I(KARR)
  INTEGER(I4B), DIMENSION(:), INTENT(IN) :: KARR
  INTEGER(I4B), DIMENSION(1) :: IMAX
  INTEGER(I4B) :: IMAXLOC_I
  IMAX=MAXLOC(KARR(:))
  IMAXLOC_I=IMAX(1)
END FUNCTION IMAXLOC_I

FUNCTION OUTERPROD_R(PA,PB)
  REAL(SP), DIMENSION(:), INTENT(IN) :: PA,PB
  REAL(SP), DIMENSION(SIZE(PA),SIZE(PB)) :: OUTERPROD_R
  OUTERPROD_R = SPREAD(PA,DIM=2,NCOPIES=SIZE(PB)) * SPREAD(PB,DIM=1,NCOPIES=SIZE(PA))
END FUNCTION OUTERPROD_R

FUNCTION OUTERPROD_D(PA,PB)
  REAL(DP), DIMENSION(:), INTENT(IN) :: PA,PB
  REAL(DP), DIMENSION(SIZE(PA),SIZE(PB)) :: OUTERPROD_D
  OUTERPROD_D = SPREAD(PA,DIM=2,NCOPIES=SIZE(PB)) * SPREAD(PB,DIM=1,NCOPIES=SIZE(PA))
END FUNCTION OUTERPROD_D

FUNCTION VABS(PV)
  REAL(SP), DIMENSION(:), INTENT(IN) :: PV
  REAL(SP) :: VABS
  VABS=SQRT(DOT_PRODUCT(PV,PV))
END FUNCTION VABS

END MODULE NRUTIL
