
USE NETCDF

IMPLICIT NONE

!$ INCLUDE 'omp_lib.h'

INCLUDE 'mpif.h'

INTEGER*4, PARAMETER :: NLON=120960, NLAT=47040, NFACT=320
INTEGER*1,DIMENSION(NLON,NFACT) :: ITAB11, ITAB12, ITAB1, ICPTAN
INTEGER*2, DIMENSION(NLON,NFACT) :: ITAB21, ITAB22
REAL*4, DIMENSION(NLON,NFACT) :: ZTAB1, ZTAB2, ZTAB
CHARACTER(LEN=100) :: YFILENAME1, YFILENAME2
INTEGER :: IFILE_ID1, IFILE_ID2, IRET, IVAR_ID1, IVAR_ID2, JLAT, JLAT2, ITYPE, JDIM, INDIMS
INTEGER, DIMENSION(2) :: ISTART, ICOUNT, IDIMLEN, IDIMIDS
INTEGER :: IAN, IMOIS, NRANK, IM1, IM2
CHARACTER(LEN=4), DIMENSION(36) :: YMOIS, YMOIS2
CHARACTER(LEN=4), DIMENSION(4) :: YAN
INTEGER :: INFOMPI

DATA YAN/"2014","2015","2016","2017"/

DATA YMOIS/"0110","0120","0131","0210","0220","0228","0310","0320","0331",&
           "0410","0420","0430","0510","0520","0531","0610","0620","0630",&
           "0710","0720","0731","0810","0820","0831","0910","0920","0930",&
           "1010","1020","1031","1110","1120","1130","1210","1220","1231"/

DATA YMOIS2/"0105","0115","0125","0205","0215","0225","0305","0315","0325",&
           "0405","0415","0425","0505","0515","0525","0605","0615","0625",&
           "0705","0715","0725","0805","0815","0825","0905","0915","0925",&
           "1005","1015","1025","1105","1115","1125","1205","1215","1225"/

ISTART(1) = 1
ICOUNT(1) = NLON
ICOUNT(2) = NFACT

CALL MPI_INIT(INFOMPI)

CALL MPI_COMM_RANK(MPI_COMM_WORLD,NRANK,INFOMPI)
IM1 = (NRANK*6) + 1 
IM2 = (NRANK*6) + 6
print*,NRANK,IM1,IM2
! pour chaque décade
DO IMOIS = IM1,IM2

  ! tableau dans lequel on va écrire le LAI pour cette décade
  OPEN(11,FILE="LAI_"//YMOIS2(IMOIS)//".bin",FORM='unformatted',ACCESS='direct',RECL=NLON)

  ! boucle sur les latitudes
!!$OMP PARALLEL DO PRIVATE(JLAT,ZTAB,ICPTAN,ISTART,IAN,YFILENAME1,YFILENAME2,IRET,IFILE_ID1,IFILE_ID2,&
!!$OMP IVAR_ID1,IVAR_ID2,ITAB11,ITAB12,ITAB21,ITAB22,ZTAB1,ZTAB2,JLAT2)
  DO JLAT = 1,NLAT/NFACT

    print*,NRANK,IMOIS,JLAT,NLAT/NFACT
    !tableau contenant la somme des années pour cette décade et cette fraction de domaine
    ZTAB(:,:) = 0.

    ! compteur des années
    ICPTAN(:,:) = 0

    ! début de lecture dans le fichier
    ISTART(1) = 1
    ISTART(2) = NFACT * (JLAT-1) + 1

    ! pour chaque année
    DO IAN = 1,SIZE(YAN)
  
      ! si la décade est en janvier (4 ans) ou si on est à moins de 3 ans)
      IF ( IMOIS<=3.OR.IAN<=3 ) THEN


        ! fichier de la décade
        YFILENAME1 = "LAI300_"//YAN(IAN)//YMOIS(IMOIS)//"0000_GLOBE_PROBAV_V1.0.1/"//&
                     "c_gls_LAI300_"//YAN(IAN)//YMOIS(IMOIS)//"0000_GLOBE_PROBAV_V1.0.1.nc"
        ! fichier de la décade suivante pour la moyenne entre les 2
        IF (IMOIS<SIZE(YMOIS)) THEN
          YFILENAME2 = "LAI300_"//YAN(IAN)//YMOIS(IMOIS+1)//"0000_GLOBE_PROBAV_V1.0.1/"//&
                       "c_gls_LAI300_"//YAN(IAN)//YMOIS(IMOIS+1)//"0000_GLOBE_PROBAV_V1.0.1.nc"
        ELSE
          YFILENAME2 = "LAI300_"//YAN(IAN+1)//YMOIS(1)//"0000_GLOBE_PROBAV_V1.0.1/"//&
                       "c_gls_LAI300_"//YAN(IAN+1)//YMOIS(1)//"0000_GLOBE_PROBAV_V1.0.1.nc"
        ENDIF

        ! on ouvre les fichiers netcdf
        IRET = NF90_OPEN(YFILENAME1,NF90_NOWRITE,IFILE_ID1)
        IRET = NF90_OPEN(YFILENAME2,NF90_NOWRITE,IFILE_ID2)
        
        ! on cherche la variables LAI
        IRET = NF90_INQ_VARID(IFILE_ID1,"LAI",IVAR_ID1)
        IRET = NF90_INQ_VARID(IFILE_ID2,"LAI",IVAR_ID2)

        ! on lit la variable LAI 
        IRET = NF90_GET_VAR(IFILE_ID1,IVAR_ID1,ITAB11,ISTART,ICOUNT)
        IRET = NF90_GET_VAR(IFILE_ID2,IVAR_ID2,ITAB12,ISTART,ICOUNT)
      
        ! fermeture des fichiers
        IRET = NF90_CLOSE(IFILE_ID1)
        IRET = NF90_CLOSE(IFILE_ID2)

        !on calcule les valeurs réelles de LAI
        ITAB21 = ITAB11
        WHERE (ITAB11<0) ITAB21 = ITAB11 + 256
        WHERE (ITAB21/=255) 
          ZTAB1 = ITAB21 * 0.033333
        ELSEWHERE
          ZTAB1 = -999.
        ENDWHERE

        ITAB22 = ITAB12
        WHERE (ITAB12<0) ITAB22 = ITAB12 + 256
        WHERE (ITAB22/=255) 
          ZTAB2 = ITAB22 * 0.033333
        ELSEWHERE
          ZTAB2 = -999.
        ENDWHERE

        ! on fait la moyenne entre 2 décade et la somme sur les années
        WHERE (ZTAB1/=-999..OR.ZTAB2/=-999.)

          ICPTAN = ICPTAN + 1

          WHERE ( ZTAB1/=-999. .AND. ZTAB2/=-999. ) 
            ZTAB = ZTAB + (ZTAB1 + ZTAB2) / 2.
          ELSEWHERE ( ZTAB1/=-999. ) 
            ZTAB = ZTAB + ZTAB1
          ELSEWHERE ( ZTAB2/=-999. ) 
            ZTAB = ZTAB + ZTAB2
          ENDWHERE
          
        ENDWHERE
   
      ENDIF

    ENDDO
    ! fin de la boucle sur les années

    !  moyenne des valeurs sur les années disponible
    WHERE (ICPTAN/=0) 
      ZTAB = ZTAB / ICPTAN
    ELSE WHERE
      ZTAB = -999.
    END WHERE

    ! on traduit en entier pour l'écriture du fichier
    WHERE (ZTAB/=-999.) 
      ITAB1 = NINT(ZTAB*10.)
    ELSEWHERE 
      ITAB1 = -1
    ENDWHERE

    ! écriture dans le fichier de sortie
    DO JLAT2 = 1,NFACT
      WRITE(11,REC=ISTART(2)+JLAT2-1) ITAB1(:,JLAT2)
    ENDDO

  ENDDO
!!$OMP END PARALLEL DO 
  ! fin de la boucle sur les fractions de domaine
  
  ! fermeture du fichier binaire  
  CLOSE(11)

ENDDO
! fin de la boucle sur les décades

CALL MPI_FINALIZE(INFOMPI)

      END
