SUBROUTINE GENE_ANALYSIS(KNT, KLIN, KCOL, PFRAC, PFIELD_ECO, PFIELD_ECO_ERR, PFIELD_SAT, PFIELD_SAT_ERR, &
                        PFIELD_GUESS, PERR_GUESS, PT, PTC, PALPHA, PFIELD_A, PERR_A, PFIELD_A_MEAN)

IMPLICIT NONE

INCLUDE 'mpif.h'

INTEGER, INTENT(IN) :: KNT ! 12
INTEGER, INTENT(IN) :: KLIN
INTEGER, INTENT(IN) :: KCOL

REAL (KIND=4), DIMENSION(KNT,KLIN,KCOL), INTENT(IN) :: PFRAC ! FRAC. TILE
REAL (KIND=4), DIMENSION(KNT,KLIN,KCOL), INTENT(IN) :: PFIELD_ECO ! ECOCLIMAP LAI. TILE
REAL (KIND=4), DIMENSION(KNT), INTENT(IN) :: PFIELD_ECO_ERR

REAL (KIND=4), DIMENSION(KLIN,KCOL), INTENT(IN) :: PFIELD_SAT   ! OBSERVED LAI FROM SAT AND QUAL FLAG- DAY J
REAL (KIND=4), INTENT(IN) :: PFIELD_SAT_ERR
!
REAL (KIND=4), DIMENSION(KNT,KLIN,KCOL), INTENT(IN) :: PFIELD_GUESS ! GUESS  LAI TILE- DAY OF THE LAST ANALYSIS

REAL (KIND=4), DIMENSION(KNT,KLIN,KCOL), INTENT(IN) :: PERR_GUESS

REAL (KIND=4), DIMENSION(KNT), INTENT(IN) :: PT
REAL (KIND=4), DIMENSION(KNT,KLIN,KCOL), INTENT(IN) :: PTC
REAL (KIND=4), DIMENSION(KNT,KLIN,KCOL), INTENT(IN) :: PALPHA


REAL (KIND=4), DIMENSION(KNT,KLIN,KCOL), INTENT(INOUT) :: PFIELD_A ! LAI TILE ANALYSED -DAY J
REAL (KIND=4), DIMENSION(KNT,KLIN,KCOL), INTENT(INOUT) :: PERR_A  ! COVAR TILE ANALYSED -DAY J
REAL (KIND=4), DIMENSION(KLIN,KCOL), INTENT(INOUT) :: PFIELD_A_MEAN   ! LAI ANALYSED - DAY J

REAL (KIND=4), DIMENSION(KNT) :: ZFRAC

!REAL (KIND=4), INTENT(IN) :: ZLAI_MOD   ! ECOCLIMAP LAI - DAY J
!REAL, PARAMETER ::ZSIG_LAI_M=.5  ! ERROR OF LAI - DAY J

REAL, DIMENSION(:,:), ALLOCATABLE :: ZA, ZB, ZR, ZH, ZK, ZT1, ZT2
REAL, DIMENSION(:), ALLOCATABLE :: ZY, ZX, ZX_OLD

INTEGER, DIMENSION(KNT,KNT) :: II_N 

INTEGER :: JT, JJ, IERR, JC, JL, NRANK, INFOMPI

! INTERFACE FOR MATRIX INVERSION
INTERFACE
  FUNCTION INVERS(PM)
    REAL, DIMENSION(:,:), INTENT(IN)  :: PM
    REAL, DIMENSION(SIZE(PM,1),SIZE(PM,1)) :: INVERS
  END FUNCTION INVERS
END INTERFACE

CALL MPI_COMM_RANK(MPI_COMM_WORLD,NRANK,INFOMPI)

IF(.NOT. ALLOCATED(ZX)) ALLOCATE(ZX(KNT))
IF(.NOT. ALLOCATED(ZX_OLD)) ALLOCATE(ZX_OLD(KNT))
IF(.NOT. ALLOCATED(ZB)) ALLOCATE(ZB(KNT,KNT))
IF(.NOT. ALLOCATED(ZA)) ALLOCATE(ZA(KNT,KNT))

IF(.NOT. ALLOCATED(ZY)) ALLOCATE(ZY(KNT+1))

IF(.NOT. ALLOCATED(ZH)) ALLOCATE(ZH  (KNT+1,KNT))
IF(.NOT. ALLOCATED(ZK)) ALLOCATE(ZK  (KNT,KNT+1))
IF(.NOT. ALLOCATED(ZT2)) ALLOCATE(ZT2(KNT,KNT+1))

IF(.NOT. ALLOCATED(ZR)) ALLOCATE(ZR  (KNT+1,KNT+1))
IF(.NOT. ALLOCATED(ZT1)) ALLOCATE(ZT1(KNT+1,KNT+1))

DO JL = 1,KLIN
  !print*,JL,KLIN  
  DO JC = 1,KCOL

    IF ( PFIELD_SAT(JL,JC)>=0.01.AND.PFIELD_SAT(JL,JC)<=1.0 .AND. &
         SUM(PFRAC(:,JL,JC))/=0. ) THEN

      !INITIALISE
      ZFRAC(1:KNT) = PFRAC(1:KNT,JL,JC)  !/SUM(PFRAC(4:KNT,JL,JC))
      !ZFRAC(1:3) = 0.
      
      !ALLOCATE X
      ! pour l'albedo, ce doit être 1 ALBVEG 2 ALBSOIL
      ZX(:) = PFIELD_GUESS(:,JL,JC)   !! DONNEES GUESS TILE SUR PIXEL
      ZX_OLD(:) = ZX(:)

      !ALLOCATE Y
      ZY(1:KNT) = PFIELD_ECO(:,JL,JC) !! DONNEES ECOCLIMAP SUR PIXEL
      ZY(KNT+1) = PFIELD_SAT(JL,JC)  !! DONNEES SAT SUR PIXEL
 
      !ALLOCATE H
      ZH(:,:) = 0.
      DO JT =1,KNT
        ZH(JT,JT) = 1
        ZH(KNT+1,JT) = ZFRAC(JT)
      ENDDO

      !ALLOCATE R
      ZR(:,:) = 0.
      DO JT = 1,KNT
        ZR(JT,JT) = PFIELD_ECO_ERR(JT)**2 !!ZSIG_LAI_M**2 # ERREUR SUR ECOCLIMAP
      ENDDO
      ZR(KNT+1,KNT+1) = PFIELD_SAT_ERR**2

      !ALLOCATE II_N
      II_N(:,:) = 0
      DO JT = 1,KNT
        II_N(JT,JT) = 1
      ENDDO
      !CALCULATE B
      ZB(:,:)=0.
      DO JT=1,KNT
        DO JJ=1,KNT

          IF (JT .NE. JJ) THEN
            ZB(JT,JJ) = 0.
          ELSE

            ZB(JT,JJ) = PERR_GUESS(JT,JL,JC) * PALPHA(JT,JL,JC) * (2**(PT(JT)/PTC(JT,JL,JC)))

            ! SECURITY LIMIT
            IF (ZFRAC(JT) .EQ. 0.)   ZB(JT,JJ) = 0.000001
            IF (ZB(JT,JJ) .GE. 100000000.) ZB(JT,JJ) = 100000000.
            IF (ZB(JT,JJ) .LE. -100000000.)   ZB(JT,JJ) = -100000000.
            IF (ABS(ZB(JT,JJ)) .LE. 0.000001)   ZB(JT,JJ) = 0.000001
          ENDIF

        ENDDO
      ENDDO
      !if (jl==4.and.jc==233) print*,"B ",ZB(1,1),PERR_GUESS(JT,JL,JC),PALPHA(JT,JL,JC),PT(:),PTC(:,JL,JC)
     
      ! SOIL AND LAIETATION ALBEDO ANALYSES

      ! (KNT+1,KNT+1) = (KNT+1,KNT) * ( (KNT,KNT) * (KNT,KNT+1) ) + (KNT+1,KNT+1)
      ZT1(:,:) = INVERS( MATMUL( ZH(:,:), MATMUL(ZB(:,:), TRANSPOSE(ZH(:,:))) ) + ZR(:,:)  )
 
      ! (KNT,KNT+1) = (KNT,KNT+1) * (KNT+1,KNT+1)
      ZT2(:,:) = MATMUL( TRANSPOSE( ZH(:,:)), ZT1(:,:) )

      ! (KNT,KNT+1) = (KNT,KNT) * (KNT,KNT+1)
      ZK(:,:) = MATMUL( ZB(:,:), ZT2(:,:) )

      ! (KNT,KNT) = ( (KNT,KNT) - (KNT,KNT+1) * (KNT+1,KNT) ) * (KNT,KNT)
      ZA(:,:)=ZB(:,:) - MATMUL( MATMUL( ZK(:,:), ZH(:,:) ), ZB(:,:))
      IF (ZA(1,1).LE.0..OR.ZA(2,2).LE.0.) THEN
        ZA(:,:) = ABS(ZB(:,:)) * 10.
      !ELSE
      ENDIF

      ! SECURITY FOR VARIANCE & LAI VALUES 
        ! KNT = KNT + ( (KNT,KNT+1) * ( (KNT+1) - (KNT+1,KNT) * KNT ))
        ZX(:) = ZX(:) + MATMUL(ZK(:,:), ( ZY(:) - MATMUL( ZH(:,:), ZX(:) ) ) )
 
      !ENDIF
      
      WHERE (ZA(:,:) .LE. -1000.**2) ZA(:,:) = -1000.**2        
      WHERE (ZA(:,:) .GE.  1000.**2) ZA(:,:) = 1000.**2        
      WHERE (ZX(:) .LT. 0.) ZX(:) = 0.

      ! OUTPUT
      !LAI TILE ANALYSED <--- X
      !Y ECO TILES & SAT <--- Y(KNT)& Y(KNT+1)

      ! CALCULATE LAI ANA TOT PIXEL
      PFIELD_A_MEAN(JL,JC) = 0.
      DO JT = 1,KNT
        ! SECURITY TESTS - SHOULD PROBABLY NOT BE ENOUGH... !!!!
        !X(I)=MIN(X(I), 0.99)
        !X(I)=MAX(X(I), 0.01)
        PFIELD_A_MEAN(JL,JC) = PFIELD_A_MEAN(JL,JC) + ZX(JT) * PFRAC(JT,JL,JC)
      ENDDO

      ! SECURITY TESTS - SHOULD PROBABLY NOT BE ENOUGH... !!!!
      DO JT = 1,KNT
        IF (PFIELD_A_MEAN(JL,JC) .GT. 1.) THEN
          ZX(JT) = ZX(JT) * 1. / PFIELD_A_MEAN(JL,JC)
        ENDIF
      ENDDO
      !IF (ZX(2)>=1.) then
      !  print*,'X ',ZX(:),ZX_OLD(:)
      !  print*,'B ',ZB(:,:)
      !  print*,'R ',ZR(:,:)
      !  print*,'err_guess ',PERR_GUESS(:,JL,JC),JL,JC
      !  print*,'alpha ',PALPHA(:,JL,JC) 
      !  print*,'pt ',PT(:)
      !  print*,'ptc ',PTC(:,JL,JC)
      !  print*,'Y ',( ZY(:) - MATMUL( ZH(:,:), ZX_OLD(:) ) )
      !ENDIF
 

      PFIELD_A_MEAN(JL,JC) = MIN(PFIELD_A_MEAN(JL,JC), 1.)       
      PFIELD_A_MEAN(JL,JC) = MAX(PFIELD_A_MEAN(JL,JC), 0.001)

      PFIELD_A (:,JL,JC) = 0.
      PERR_A(:,JL,JC) = 0.
      DO JT = 1,KNT

        !IF (ZA(JT,JT)==0.) then
        !  print*,"ZA 0 ",JT 
        !endif
        ZX(JT) = MIN(ZX(JT), 1.)
        ZX(JT) = MAX(ZX(JT), 0.001)
        PFIELD_A (JT,JL,JC) = ZX(JT)
        PERR_A(JT,JL,JC) = ZA(JT,JT)

      ENDDO
    
    ELSE

      PFIELD_A(:,JL,JC) = PFIELD_ECO(:,JL,JC)
      PERR_A(:,JL,JC) = PERR_GUESS(:,JL,JC)*(2**(2*PT(:)/PTC(:,JL,JC)))
      WHERE (PERR_A(:,JL,JC).GE.100000000.) PERR_A(:,JL,JC) = 100000000.
      PFIELD_A_MEAN(JL,JC) = 0.
      DO JT = 1,KNT
        PFIELD_A_MEAN(JL,JC) = PFIELD_A_MEAN(JL,JC) + PFIELD_A(JT,JL,JC) * PFRAC(JT,JL,JC)
      ENDDO
      
    ENDIF

  ENDDO
ENDDO
       
! DEALLOCATE
DEALLOCATE( ZY, ZX, ZA, ZB, ZR, ZH, ZK, ZT1, ZT2, STAT=IERR )

IF (IERR .NE. 0) THEN
  PRINT *,'ERROR WHEN DEALLOCATING VARIABLES PACKAGE LAI!'
  RETURN
END IF        

END SUBROUTINE GENE_ANALYSIS

